use std::collections::VecDeque;

use cpal::{FromSample, Sample};

use tracing::trace;

use crate::dsp_common::carrier::{Carrier, TwoStateCarrier};

pub fn give_output_pcm<T>(
    pcm: &mut [T],
    rx_for_mod: &mut tokio::sync::mpsc::Receiver<u8>,
    carrier: &mut TwoStateCarrier,
    leftovers: &mut VecDeque<f64>,
) where
    T: Sample + std::fmt::Debug + FromSample<f64>,
{
    trace!("ALSA demands {} samples...", pcm.len());
    let mut output = pcm.iter_mut().peekable();

    while let Some(samp) = leftovers.pop_front() {
        if let Some(sample_to_write) = output.next() {
            *sample_to_write = samp.to_sample::<T>();
        } else {
            trace!("ALSA demand fulfilled (last samp with leftovers), {} leftovers remain", leftovers.len());
            return;
        }
    }

    while let Ok(byte) = rx_for_mod.try_recv() {
        let samps = carrier.byte_into_fsk_samples(&byte);
        //trace!("new ether samps {:?}", samps);
        let mut samps_iter = samps.iter().cloned();
        while let Some(samp) = samps_iter.next() {
            if let Some(sample_to_write) = output.next() {
                *sample_to_write = samp.to_sample::<T>();
            } else {
                let mut collected = samps_iter.collect::<VecDeque<f64>>();
                leftovers.append(&mut collected);
                trace!("ALSA demand fulfilled (last samp with modulated eth data)! collected {} leftovers", leftovers.len());
                return;
            }
        }
    }

    while let Some(_) = output.peek() {
        let idle_samps = carrier.idle();
        let mut idle_samps_iter = idle_samps.iter().cloned();
        while let Some(samp) = idle_samps_iter.next() {
            if let Some(sample_to_write) = output.next() {
                *sample_to_write = samp.to_sample::<T>();
            } else {
                let mut collected = idle_samps_iter.collect::<VecDeque<f64>>();
                leftovers.append(&mut collected);
                trace!("ALSA demand fulfilled (last samp with empty carrier)! collected {} leftovers", leftovers.len());
                return;
            }
        }
    }

    trace!("Alsa demand fulfilled with no leftovers, apparently. (leftovers={})", leftovers.len());
}
